/*
 *	Network Queueing System (NQS)
 *  This version of NQS is Copyright (C) 1992  John Roman
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
*  PROJECT:     Network Queueing System
*  AUTHOR:      John Roman
*
*  Modification history:
*
*       Version Who     When            Description
*       -------+-------+---------------+-------------------------
*	V01.1	JRR	26-May-1992    	Initial version
*	     	JRR	17-Jun-1992	Added header
*	     	JRR	21-Sep-1992	Added startup and shutdown processing.
*	     	JRR	07-Oct-1992	Started qacct.c
*	V01.2   JRR	15-Jan-1993	Fixed month offset.
*					Fixed -u processing.
*/
/*++ qacct.c - Network Queueing System
 *

 * DESCRIPTION:
 *
 *	Dump the NQS accounting file.
 *
 *
 *	Author:
 *	-------
 *	John Roman,  Monsanto Company.
 *	May 26,  1992.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 */

#define VERSION "V3.31 -- January, 1993"

#include <errno.h>
#include "nqs.h"			/* NQS constants and data types */
#include "nqsxvars.h"			/* NQS global variables */
#include "nqsacct.h"
#include <time.h>
#include <malloc.h>
#include <sys/stat.h>
#include <fcntl.h>



struct acct_record {
	struct acct_record *next;
        char    user[16];               /* user name */
        char    queue[16];              /* NQS queue name */
        int     priority;               /* priority of NQS queue */
        time_t  sub_time;               /* time submitted */
        time_t  start_time;             /* requested start time "-a" */
        time_t  init_time;              /* time initiated */
        time_t  fin_time;               /* Time completed */
        int     orig_mid;               /* originating machine ID */
        char    scp_tid[8];             /* station TID */
        char    scp_mf[2];              /* station mainframe ID */
        long    seqno;                  /* original sequence number */
#if    UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
        long    tms_stime;              /* Time spent by system for this job */ 
	long    tms_utime;              /* Time spent by user for this job */
#else
#if     BSD42 | BSD43 | ULTRIX
        long    s_sec;                  /* System time in seconds */
        long    s_usec;                 /* System time in microseconds */
        long    u_sec;                  /* User time in seconds */
        long    u_usec;                 /* User time in microseconds */
#else
BAD SYSTEM TYPE
#endif
#endif
};

#ifndef __CEXTRACT__
#if __STDC__

static print_outstanding_req ( struct acct_record *arp, char *type );
static qacct_sho_version ( void );
static qacct_showhow ( void );
static void bad_syntax ( char *flag );
static void bad_value ( char *flag );
static print_partial_req ( struct acct_record *arp );

#else /* __STDC__ */

static print_outstanding_req (/* struct acct_record *arp, char *type */);
static qacct_sho_version (/* void */);
static qacct_showhow (/* void */);
static void bad_syntax (/* char *flag */);
static void bad_value (/* char *flag */);
static print_partial_req (/* struct acct_record *arp */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */

/*** main
 *
 *  qacct [-v] [-u username] [-a after-time] [-b before-time] [-c] [-f]
 *
 *	    -v		prints version
 *	    -u username prints records for that username
 *	    -a time	prints records completed since that time
 *	    -b time     prints records completed before that time
 *	    -c		consolidates the file by deleting closed records
 *	    -f		prints everything.	
 */
main (argc, argv)
int argc;
char *argv[];                           
{
    char *fmtmidname();
    register int i;
    int fd_acct;                      /* Accounting file descriptor */
    struct nqsacct_fin acct_fin;      /* Accounting structure to report */
                                      /* Cpu usage                      */
    struct nqsacct_fin1 acct_fin1;    /* Accounting structure to report */
                                      /* Cpu usage                      */
    struct nqsacct_init acct_init;
    struct nqsacct_startup acct_startup;
    struct nqsacct_shutdown acct_shutdown;
    struct nqsacct	acct_header;
    int status;
    char ctime_buffer[32];
    struct acct_record acct_list_head;
    struct acct_record *arp;
    struct acct_record *arp_prev;
    int nqs_is_up;		       /* Logical  -- assumed status of NQS */
    struct tm *tp;
    char *whom;
    int	full;
    char *argument;
    double cpu_time;
    time_t after_time;
    time_t before_time;
	
    nqs_is_up = 0;			/* Assume starting down */
    
    whom = NULL;
    full = 0;
    after_time = 0;
    before_time = -1;
    while (*++argv != NULL && **argv == '-') {
        argument = *argv;
        while (*++argument != '\0') {
            switch (*argument) {
                case 'f':
                    full++;
                    break;
                case 'v':
                    qacct_sho_version();
                    exit(0);
                    break;
                case 'a':               /* After time specification */
                    if (*++argv == NULL) {
                        fprintf (stderr, "Missing after-time.\n");
                        exit (-1);
                    }
                    if (after_time != NULL) {
                        fprintf (stderr, "Multiple -a specifications.\n");
                        exit (-1);
                    }
                    switch (scnftime (*argv, &after_time)) {
                        case 0:
                            break;
                        case -1:
                             bad_syntax ("a"); /* Exit */
                        case -2:
                        case -3:
                             bad_value ("a");   /* Exit */
                        }
                    break;
                case 'b':               /* Before time specification */
                    if (*++argv == NULL) {
                        fprintf (stderr, "Missing before-time.\n");
                        exit (-1);
                    }
                    if (before_time != -1) {
                        fprintf (stderr, "Multiple -b specifications.\n");
                        exit (-1);
                    }
                    switch (scnftime (*argv, &before_time)) {
                        case 0:
                            break;
                        case -1:
                             bad_syntax ("b"); /* Exit */
                        case -2:
                        case -3:
                             bad_value ("b");   /* Exit */
                        }
                    break;
                case 'u':               /* User-name specification */
                    if (*++argv == NULL) {
                        fprintf (stderr, "Missing username.\n");
                        exit (-1);
                    }
                    if (whom != NULL) {
                        fprintf (stderr, "Multiple -u specifications.\n");
                        exit (-1);
                    }
                    whom = *argv;
                    break;
                default:
                    fprintf (stderr, "Invalid option flag ");
                    fprintf (stderr, "specified.\n");
                    qacct_showhow();
                    exit (7);
            }
        }
    }
    
    bzero ( acct_list_head,  sizeof(acct_list_head) );
    fd_acct = open(NQSACCT_FILE, O_RDONLY, 0644);
    if (fd_acct < 0) {
	fprintf (stderr, "Error opening NQS account");
        fprintf (stderr, " file;  Errno = %d\n", errno);
        fflush (stderr);
	exit(1);
     }
     while (1) {
        status = read (fd_acct,  &acct_header,  sizeof(acct_header));
	if (status == 0 ) break;	/* End of file encountered */
	if (status != sizeof(acct_header) ) {
	    fprintf(stderr,  "Error reading NQS account file;  Errno = %d\n", 
		errno);
	    exit(1);
	}
	/*
	 * ==================================================================
	 */
        if (acct_header.type == NQSACCT_FIN) {
	    status = read (fd_acct,  &acct_fin.user,  
		    acct_header.length-sizeof(acct_header));
	    if (status == 0 ) break;	/* End of file encountered */
	    if (status != sizeof(acct_fin)-sizeof(acct_header) ) {
		fprintf(stderr,  "Error reading NQS account file;  Errno = %d\n", 
		    errno);
		exit(1);
	    }
	    if (full) {
	         printf(" TERM request %d.%s\n",acct_fin.seqno,  
		    fmtmidname(acct_fin.orig_mid));
	    }
	    arp = acct_list_head.next;
	    arp_prev = &acct_list_head;
	    while (1) {
		if (arp == NULL) {
		    if (full) {
		        printf("Found term but no init for:\n");
		        printf(" %s",  acct_fin.user);
		        printf(" request %d.%s",
				acct_fin.seqno,  
				fmtmidname(acct_fin.orig_mid));
		        printf(" on %s\n",  acct_fin.queue);
#if     UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
		        printf(" System time = %d", acct_fin.tms_stime);
		        printf(" User time = %d\n",  acct_fin.tms_utime); 
#else
#if     BSD42 | BSD43 | ULTRIX
		        printf("    System time = %d, %d\n",acct_fin.s_sec, acct_fin.s_usec);
		        printf("    User time = %d,  %d\n",acct_fin.u_sec, acct_fin.u_usec);
#else
BAD SYSTEM TYPE
#endif
#endif
		    }
		    break;
		}
		if (arp->seqno == acct_fin.seqno 
			&& arp->orig_mid == acct_fin.orig_mid) break;
		arp_prev = arp;
		arp = arp->next;
	    }
	    if (arp == NULL) continue;
	    if ( ( whom == NULL) || (whom != NULL && !strncmp(whom, acct_fin.user, 
			    strlen(whom))) ){
	         printf("%-8.8s",  acct_fin.user);
	         printf("%6d.%-8.8s",
		    acct_fin.seqno,  
		    fmtmidname(acct_fin.orig_mid));
	         printf(" %-9.9s",  acct_fin.queue);
#if     UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
#if	IBMRS
		cpu_time = (double) (acct_fin.tms_stime+acct_fin.tms_utime);
		printf(" %9.2f",  cpu_time/100.0);
#else
	         printf(" %9d", acct_fin.tms_stime+acct_fin.tms_utime);
#endif
#else
#if     BSD42 | BSD43 | ULTRIX
	         printf(" %9d",
		    acct_fin.s_sec+acct_fin.u_sec);
#else
BAD SYSTEM TYPE
#endif
#endif
	         tp = localtime(&arp->init_time);
	         printf(" %d-%2.2d-%2.2d:%2.2d:%2.2d:%2.2d\n", tp->tm_year,  
			tp->tm_mon+1,  tp->tm_mday, 
			tp->tm_hour,  tp->tm_min,  tp->tm_sec);
	    }
	    arp_prev->next = arp->next;
	    free (arp); 
	/*
	 * =====================================================================
	 */   
        } else if (acct_header.type == NQSACCT_FIN_1) {
	    /*
	     * Completion records type 1:
	     *	    These have the completion time in the record.
	     */
	    status = read (fd_acct,  &acct_fin1.user,  
		    acct_header.length-sizeof(acct_header));
	    if (status == 0 ) break;	/* End of file encountered */
	    if (status != sizeof(acct_fin1)-sizeof(acct_header) ) {
		fprintf(stderr,  "Error reading NQS account file;  Errno = %d\n", 
		    errno);
		exit(1);
	    }
	    if (full) {
	         printf(" TERM_1 request %d.%s ",acct_fin1.seqno,  
		    fmtmidname(acct_fin1.orig_mid));
		    tp = localtime(&acct_fin1.fin_time);
		    printf(" 19%d-%2.2d-%2.2d:%2.2d:%2.2d:%2.2d\n", tp->tm_year,  
			tp->tm_mon+1,  tp->tm_mday, 
			tp->tm_hour,  tp->tm_min,  tp->tm_sec);
	    }
	    arp = acct_list_head.next;
	    arp_prev = &acct_list_head;
	    while (1) {
		if (arp == NULL) {
		    if (full) {
		        printf("Found term1 but no init for:\n");
		        printf(" %s",  acct_fin1.user);
		        printf(" request %d.%s",
				acct_fin1.seqno,  
				fmtmidname(acct_fin1.orig_mid));
		        printf(" on %s\n",  acct_fin1.queue);
#if     UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
		        printf(" System time = %d", acct_fin1.tms_stime);
		        printf(" User time = %d\n",  acct_fin1.tms_utime); 
#else
#if     BSD42 | BSD43 | ULTRIX
		        printf("    System time = %d, %d\n",acct_fin1.s_sec, acct_fin1.s_usec);
		        printf("    User time = %d,  %d\n",acct_fin1.u_sec, acct_fin1.u_usec);
#else
BAD SYSTEM TYPE
#endif
#endif
		    }
		    
		    break;
		}
		if (arp->seqno == acct_fin1.seqno 
			&& arp->orig_mid == acct_fin1.orig_mid) break;
		arp_prev = arp;
		arp = arp->next;
	    }
	    if (arp == NULL) continue;
	    if ( ( whom == NULL) || (whom != NULL && !strncmp(whom, acct_fin1.user, 
			    strlen(whom))) ){
		/* if (acct_fin1.fin_time >= after_time && 
			    acct_fin1.fin_time <= before_time) { */
	            printf("%-8.8s",  acct_fin1.user);
	            printf("%6d.%-8.8s", acct_fin1.seqno, fmtmidname(acct_fin1.orig_mid));
		    printf(" %-9.9s",  acct_fin1.queue);
#if     UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
#if	IBMRS
		    cpu_time = (double) (acct_fin1.tms_stime+acct_fin1.tms_utime);
		    printf(" %9.2f",  cpu_time/100.0);
#else
		    printf(" %9d", acct_fin1.tms_stime+acct_fin1.tms_utime);
#endif
#else
#if     BSD42 | BSD43 | ULTRIX
		    printf(" %9d",
			acct_fin1.s_sec+acct_fin1.u_sec);
#else
BAD SYSTEM TYPE
#endif
#endif
		    tp = localtime(&arp->init_time);
		    printf(" %d-%2.2d-%2.2d:%2.2d:%2.2d:%2.2d", tp->tm_year,  
			tp->tm_mon+1,  tp->tm_mday, 
			tp->tm_hour,  tp->tm_min,  tp->tm_sec);
		    tp = localtime(&acct_fin1.fin_time);
		    printf(" %d-%2.2d-%2.2d:%2.2d:%2.2d:%2.2d\n", tp->tm_year,  
			tp->tm_mon+1,  tp->tm_mday, 
			tp->tm_hour,  tp->tm_min,  tp->tm_sec);
		/* } */
	    }
	    arp_prev->next = arp->next;
	    free (arp);    
	/*
	 * =====================================================================
	 */   
	} else if (acct_header.type == NQSACCT_INIT) {
	    status = read (fd_acct,  &acct_init.user,  
		    acct_header.length-sizeof(acct_header));
	    if (status == 0 ) break;	/* End of file encountered */
	    if (status != sizeof(acct_init)-sizeof(acct_header) ) {
		fprintf(stderr,  "Error reading NQS account file;  Errno = %d\n", 
		errno);
		exit(1);
	    }
	    arp = malloc(sizeof(acct_list_head) );
	    bzero ( arp,  sizeof(acct_list_head) );
	    strcpy(arp->user,  acct_init.user);
	    if (full) {
	         printf(" INIT request %d.%s ",acct_init.seqno,  
		    fmtmidname(acct_init.orig_mid));
		    tp = localtime(&acct_init.init_time);
		    printf(" 19%d-%2.2d-%2.2d:%2.2d:%2.2d:%2.2d\n", tp->tm_year,  
			tp->tm_mon+1,  tp->tm_mday, 
			tp->tm_hour,  tp->tm_min,  tp->tm_sec);
	    }
	    arp->seqno = acct_init.seqno;
	    arp->orig_mid = acct_init.orig_mid;
	    strcpy(arp->queue,  acct_init.queue);
	    arp->sub_time = acct_init.sub_time;
	    arp->init_time = acct_init.init_time;
	    arp->next = acct_list_head.next;		
	    acct_list_head.next = arp;
	/*
	 * =====================================================================
	 */   
        } else if (acct_header.type == NQSACCT_STAT) {
	    /*
	     * Status record:
	     *	    These have the same format as TERM_1 records
	     *	    but may be followed by a TERM_1 record.
	     */
	    status = read (fd_acct,  &acct_fin1.user,  
		    acct_header.length-sizeof(acct_header));
	    if (status == 0 ) break;	/* End of file encountered */
	    if (status != sizeof(acct_fin1)-sizeof(acct_header) ) {
		fprintf(stderr,  "Error reading NQS account file;  Errno = %d\n", 
		    errno);
		exit(1);
	    }
	    if (full) {
	         printf(" STAT request %d.%s ",acct_fin1.seqno,  
		    fmtmidname(acct_fin1.orig_mid));
#if     UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
		 printf(" System time = %d", acct_fin1.tms_stime);
		 printf(" User time = %d",  acct_fin1.tms_utime); 
#else
#if     BSD42 | BSD43 | ULTRIX
		 printf(" System time = %d, %d",acct_fin1.s_sec, acct_fin1.s_usec);
		 printf(" User time = %d,  %d",acct_fin1.u_sec, acct_fin1.u_usec);
#else
BAD SYSTEM TYPE
#endif
#endif		    
		 tp = localtime(&acct_fin1.fin_time);
		 printf(" 19%d-%2.2d-%2.2d:%2.2d:%2.2d:%2.2d\n", tp->tm_year,  
			tp->tm_mon+1,  tp->tm_mday, 
			tp->tm_hour,  tp->tm_min,  tp->tm_sec);
	    }
	    arp = acct_list_head.next;
	    while (1) {
		if (arp == NULL) {
		    if (full) {
		        printf("Found stat but no init for:\n");
		        printf(" %s",  acct_fin1.user);
		        printf(" request %d.%s",
				acct_fin1.seqno,  
				fmtmidname(acct_fin1.orig_mid));
		        printf(" on %s\n",  acct_fin1.queue);
#if     UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
		        printf(" System time = %d", acct_fin1.tms_stime);
		        printf(" User time = %d\n",  acct_fin1.tms_utime); 
#else
#if     BSD42 | BSD43 | ULTRIX
		        printf("    System time = %d, %d\n",acct_fin1.s_sec, acct_fin1.s_usec);
		        printf("    User time = %d,  %d\n",acct_fin1.u_sec, acct_fin1.u_usec);
#else
BAD SYSTEM TYPE
#endif
#endif
		    }
		    break;
		}
		if (arp->seqno == acct_fin1.seqno 
			&& arp->orig_mid == acct_fin1.orig_mid) break;
		arp = arp->next;
	    }
	    if (arp == NULL) continue;
	    if ( ( whom == NULL) || (whom != NULL && !strncmp(whom, acct_fin1.user, 
			strlen(whom) )) ){
#if     UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
		    arp->tms_stime = acct_fin1.tms_stime;
		    arp->tms_utime = acct_fin1.tms_utime;
#else
#if     BSD42 | BSD43 | ULTRIX
		    arp->s_sec = acct_fin1.s_sec;
		    arp->s_usec = acct_fin1.s_usec;
		    arp->u_sec = acct_fin1.u_sec;
		    arp->u_usec = acct_fin1.u_usec;
#else
BAD SYSTEM TYPE
#endif
#endif
		    arp->fin_time = acct_fin1.fin_time;
	    }

	/*
	 * =====================================================================
	 */   
	} else if (acct_header.type == NQSACCT_STARTUP) {	    
	    status = read (fd_acct,  &acct_startup.start_time,  
		    acct_header.length-sizeof(acct_header));
	    if (status == 0 ) break;	/* End of file encountered */
	    if (status != sizeof(acct_startup)-sizeof(acct_header) ) {
		fprintf(stderr,  "Error reading NQS account file;  Errno = %d\n", 
		    errno);
		exit(1);
	    }
	    print_partial_req(&acct_list_head);
	    if (full) {
		printf ("     Startup at %s", 
		      ctime( (time_t *) &acct_startup.start_time) );
		fflush(stdout);	
	        print_outstanding_req(&acct_list_head, "Startup");
	    }	
	/*
	 * =====================================================================
	 */   
	} else if (acct_header.type == NQSACCT_SHUTDOWN) {
	    status = read (fd_acct,  &acct_shutdown.down_time,  
		    acct_header.length-sizeof(acct_header));
	    if (status == 0 ) break;	/* End of file encountered */
	    if (status != sizeof(acct_shutdown)-sizeof(acct_header) ) {
		fprintf(stderr,  "Error reading NQS account file;  Errno = %d\n", 
		    errno);
		exit(1);
	    }
	    print_partial_req(&acct_list_head);
	    if (full) {
	         printf ("    Shutdown at %s",
			    ctime( (time_t *) &acct_shutdown.down_time) );
	         print_outstanding_req(&acct_list_head, "Shutdown");
	    }	
	/*
	 * =====================================================================
	 */   
	} else {
	    printf ("Unknown accounting record\n");
	    printf ("Type = %d\n",  acct_header.type);
	    exit (1);
	}
    }
    if (full) {
	print_outstanding_req(&acct_list_head, "Present");
    }
    close(fd_acct);
}
/*** print_outstanding_req
 * 
 *
 * Print out any records that have an init,  but no stat or term.
 */
static print_outstanding_req(arp, type)
struct acct_record *arp;
char *type;
{
    struct acct_record *sa_arp, *arp_ptr;
    int  header_printed = 0;

    sa_arp = arp->next;
    arp->next = NULL;
    while (sa_arp != NULL) {
        if (!header_printed) {
            printf("Outstanding requests at %s:\n", type);
            header_printed = 1;
        }
        printf("  Seqno: %d.%s",  sa_arp->seqno, fmtmidname(sa_arp->orig_mid));        printf(" %s", sa_arp->user);
        printf ("   started at %s",
                             ctime( (time_t *) &sa_arp->init_time) );
        arp_ptr = sa_arp;
        sa_arp = sa_arp->next;
        free (arp_ptr);
    }
}
/*** qacct_show_version:
 *
 * Print the current version of qacct.
 */
static qacct_sho_version()
{
     fprintf(stderr, "Qacct version is %s\n", VERSION);
}
/*** qacct_showhow:
 *
 * Print the qacct syntax.
 */
static qacct_showhow()
{
    fprintf (stderr, "Command format is:\n\n");
    fprintf (stderr, "    qacct [-f] [-v] [-u username] \n");
    exit (0);
}
/*** bad_syntax
 *
 *
 *      void bad_syntax():
 *      Display message and exit.
 */
static void bad_syntax (flag)
char *flag;                             /* Which flag */
{
        fprintf (stderr, "Invalid syntax following -%s flag.\n", flag);
        exit(0);                    /* Exit */
}


/*** bad_value
 *
 *
 *      void bad_value ():
 *      Display message and exit.
 */
static void bad_value (flag)
char *flag;                             /* Which flag */
{
        fprintf (stderr, "Invalid value following -%s flag.\n", flag);
        exit(0);                    /* Exit */
}
/*** print_partial_req
 * 
 *
 * Print all partial requests -- the ones for which we have an init but
 * no termination record (yet have seen a status).
 */
static print_partial_req(arp)
struct acct_record *arp;
{
    struct acct_record *sa_arp, *arp_ptr, *arp_prev;
    unsigned long cpu_time;
    struct tm   *tp;
    
    arp_prev = arp;
    sa_arp = arp->next;
    /*
     * Print out all that have stats w/o terms.
     */
    while (sa_arp != NULL) {
	if (sa_arp->fin_time != 0) {
	    printf("%-8.8s",  sa_arp->user);
	    printf("%6d.%-8.8s", sa_arp->seqno, fmtmidname(sa_arp->orig_mid));
	    printf(" %-9.9s",  sa_arp->queue);
#if     UNICOS | HPUX | SGI | SYS52 | UTS | IBMRS
#if	IBMRS
	    cpu_time = (double) (sa_arp->tms_stime+sa_arp->tms_utime);
	    printf(" %9.2f", cpu_time/100.0);
#else
	    printf(" %9d", sa_arp->tms_stime+sa_arp->tms_utime);
#endif
#else
#if     BSD42 | BSD43 | ULTRIX
	    printf(" %9d", sa_arp->s_sec+sa_arp->u_sec);
#else
BAD SYSTEM TYPE
#endif
#endif
	    tp = localtime(&sa_arp->init_time);
	    printf(" %d-%2.2d-%2.2d:%2.2d:%2.2d:%2.2d", tp->tm_year,  
			tp->tm_mon+1, tp->tm_mday, 
			tp->tm_hour, tp->tm_min, tp->tm_sec);
	    tp = localtime(&sa_arp->fin_time);
	    printf(" %d-%2.2d-%2.2d:%2.2d:%2.2d:%2.2d\n", tp->tm_year,  
			tp->tm_mon+1, tp->tm_mday, 
			tp->tm_hour, tp->tm_min, tp->tm_sec);
	    arp_prev->next = sa_arp->next;
 	    arp_ptr = sa_arp;
	    sa_arp = sa_arp->next;
	    free (arp_ptr);
	} else {
	    arp_prev = sa_arp;
	    sa_arp = sa_arp->next;
	}
   }
}
